import { __esmMin } from "../rolldown-runtime.js";
import { AST_NODE_TYPES, ast_exports, canTokensBeAdjacent, createRule, getPrecedence, getStaticPropertyName, init_ast, init_create_rule, isDecimalInteger, isKeywordToken, isMixedLogicalAndCoalesceExpressions, isSingleLine, isTopLevelExpressionStatement, skipChainExpression } from "../utils.js";
var no_extra_parens_default;
var init_no_extra_parens = __esmMin(() => {
	init_ast();
	init_create_rule();
	no_extra_parens_default = createRule({
		name: "no-extra-parens",
		meta: {
			type: "layout",
			docs: { description: "Disallow unnecessary parentheses" },
			fixable: "code",
			schema: { anyOf: [{
				type: "array",
				items: [{
					type: "string",
					enum: ["functions"]
				}],
				minItems: 0,
				maxItems: 1
			}, {
				type: "array",
				items: [{
					type: "string",
					enum: ["all"]
				}, {
					type: "object",
					properties: {
						conditionalAssign: { type: "boolean" },
						ternaryOperandBinaryExpressions: { type: "boolean" },
						nestedBinaryExpressions: { type: "boolean" },
						returnAssign: { type: "boolean" },
						ignoreJSX: {
							type: "string",
							enum: [
								"none",
								"all",
								"single-line",
								"multi-line"
							]
						},
						enforceForArrowConditionals: { type: "boolean" },
						enforceForSequenceExpressions: { type: "boolean" },
						enforceForNewInMemberExpressions: { type: "boolean" },
						enforceForFunctionPrototypeMethods: { type: "boolean" },
						allowParensAfterCommentPattern: { type: "string" },
						nestedConditionalExpressions: { type: "boolean" },
						allowNodesInSpreadElement: {
							type: "object",
							properties: {
								ConditionalExpression: { type: "boolean" },
								LogicalExpression: { type: "boolean" },
								AwaitExpression: { type: "boolean" }
							},
							additionalProperties: false
						}
					},
					additionalProperties: false
				}],
				minItems: 0,
				maxItems: 2
			}] },
			messages: { unexpected: "Unnecessary parentheses around expression." }
		},
		defaultOptions: ["all"],
		create(context) {
			const sourceCode = context.sourceCode;
			const tokensToIgnore = /* @__PURE__ */ new WeakSet();
			const precedence = getPrecedence;
			const ALL_NODES = context.options[0] !== "functions";
			const EXCEPT_COND_ASSIGN = ALL_NODES && context.options[1] && context.options[1].conditionalAssign === false;
			const EXCEPT_COND_TERNARY = ALL_NODES && context.options[1] && context.options[1].ternaryOperandBinaryExpressions === false;
			const NESTED_BINARY = ALL_NODES && context.options[1] && context.options[1].nestedBinaryExpressions === false;
			const EXCEPT_RETURN_ASSIGN = ALL_NODES && context.options[1] && context.options[1].returnAssign === false;
			const IGNORE_JSX = ALL_NODES && context.options[1] && context.options[1].ignoreJSX;
			const IGNORE_ARROW_CONDITIONALS = ALL_NODES && context.options[1] && context.options[1].enforceForArrowConditionals === false;
			const IGNORE_SEQUENCE_EXPRESSIONS = ALL_NODES && context.options[1] && context.options[1].enforceForSequenceExpressions === false;
			const IGNORE_NEW_IN_MEMBER_EXPR = ALL_NODES && context.options[1] && context.options[1].enforceForNewInMemberExpressions === false;
			const IGNORE_FUNCTION_PROTOTYPE_METHODS = ALL_NODES && context.options[1] && context.options[1].enforceForFunctionPrototypeMethods === false;
			const ALLOW_PARENS_AFTER_COMMENT_PATTERN = ALL_NODES && context.options[1] && context.options[1].allowParensAfterCommentPattern;
			const ALLOW_NESTED_TERNARY = ALL_NODES && context.options[1] && context.options[1].nestedConditionalExpressions === false;
			const ALLOW_NODES_IN_SPREAD = ALL_NODES && context.options[1] && new Set(Object.entries(context.options[1].allowNodesInSpreadElement || {}).filter(([_, value]) => value).map(([key]) => key));
			const PRECEDENCE_OF_ASSIGNMENT_EXPR = precedence({ type: "AssignmentExpression" });
			const PRECEDENCE_OF_UPDATE_EXPR = precedence({ type: "UpdateExpression" });
			let reportsBuffer;
			/**
			* Finds the path from the given node to the specified ancestor.
			* @param node First node in the path.
			* @param ancestor Last node in the path.
			* @returns Path, including both nodes.
			* @throws {Error} If the given node does not have the specified ancestor.
			*/
			function pathToAncestor(node, ancestor) {
				const path = [node];
				let currentNode = node;
				while (currentNode !== ancestor) {
					currentNode = currentNode.parent;
					/* c8 ignore start */
					if (currentNode === null || currentNode === void 0) throw new Error("Nodes are not in the ancestor-descendant relationship.");
					/* c8 ignore stop */
					path.push(currentNode);
				}
				return path;
			}
			/**
			* Finds the path from the given node to the specified descendant.
			* @param node First node in the path.
			* @param descendant Last node in the path.
			* @returns Path, including both nodes.
			* @throws {Error} If the given node does not have the specified descendant.
			*/
			function pathToDescendant(node, descendant) {
				return pathToAncestor(descendant, node).reverse();
			}
			/**
			* Checks whether the syntax of the given ancestor of an 'in' expression inside a for-loop initializer
			* is preventing the 'in' keyword from being interpreted as a part of an ill-formed for-in loop.
			* @param node Ancestor of an 'in' expression.
			* @param child Child of the node, ancestor of the same 'in' expression or the 'in' expression itself.
			* @returns True if the keyword 'in' would be interpreted as the 'in' operator, without any parenthesis.
			*/
			function isSafelyEnclosingInExpression(node, child) {
				switch (node.type) {
					case "ArrayExpression":
					case "ArrayPattern":
					case "BlockStatement":
					case "ObjectExpression":
					case "ObjectPattern":
					case "TemplateLiteral": return true;
					case "ArrowFunctionExpression":
					case "FunctionExpression": return node.params.includes(child);
					case "CallExpression":
					case "NewExpression": return node.arguments.includes(child);
					case "MemberExpression": return node.computed && node.property === child;
					case "ConditionalExpression": return node.consequent === child;
					default: return false;
				}
			}
			/**
			* Starts a new reports buffering. Warnings will be stored in a buffer instead of being reported immediately.
			* An additional logic that requires multiple nodes (e.g. a whole subtree) may dismiss some of the stored warnings.
			*/
			function startNewReportsBuffering() {
				reportsBuffer = {
					upper: reportsBuffer,
					inExpressionNodes: [],
					reports: []
				};
			}
			/**
			* Ends the current reports buffering.
			*/
			function endCurrentReportsBuffering() {
				const { upper, inExpressionNodes, reports } = reportsBuffer ?? {};
				if (upper) {
					upper.inExpressionNodes.push(...inExpressionNodes ?? []);
					upper.reports.push(...reports ?? []);
				} else reports?.forEach(({ finishReport }) => finishReport());
				reportsBuffer = upper;
			}
			/**
			* Checks whether the given node is in the current reports buffer.
			* @param node Node to check.
			* @returns True if the node is in the current buffer, false otherwise.
			*/
			function isInCurrentReportsBuffer(node) {
				return reportsBuffer?.reports.some((r) => r.node === node);
			}
			/**
			* Removes the given node from the current reports buffer.
			* @param node Node to remove.
			*/
			function removeFromCurrentReportsBuffer(node) {
				if (reportsBuffer) reportsBuffer.reports = reportsBuffer.reports.filter((r) => r.node !== node);
			}
			/**
			* Determines whether the given node is a `call` or `apply` method call, invoked directly on a `FunctionExpression` node.
			* Example: function(){}.call()
			* @param node The node to be checked.
			* @returns True if the node is an immediate `call` or `apply` method call.
			* @private
			*/
			function isImmediateFunctionPrototypeMethodCall(node) {
				const callNode = skipChainExpression(node);
				if (callNode.type !== "CallExpression") return false;
				const callee = skipChainExpression(callNode.callee);
				return callee.type === "MemberExpression" && callee.object.type === "FunctionExpression" && ["call", "apply"].includes(getStaticPropertyName(callee));
			}
			/**
			* Determines if this rule should be enforced for a node given the current configuration.
			* @param node The node to be checked.
			* @returns True if the rule should be enforced for this node.
			* @private
			*/
			function ruleApplies(node) {
				if (node.type === "JSXElement" || node.type === "JSXFragment") switch (IGNORE_JSX) {
					case "all": return false;
					case "multi-line": return isSingleLine(node);
					case "single-line": return !isSingleLine(node);
					case "none": break;
				}
				if (node.type === "SequenceExpression" && IGNORE_SEQUENCE_EXPRESSIONS) return false;
				if (isImmediateFunctionPrototypeMethodCall(node) && IGNORE_FUNCTION_PROTOTYPE_METHODS) return false;
				return ALL_NODES || node.type === "FunctionExpression" || node.type === "ArrowFunctionExpression";
			}
			/**
			* Determines if a node is surrounded by parentheses.
			* @param node The node to be checked.
			* @returns True if the node is parenthesised.
			* @private
			*/
			function isParenthesised(node) {
				return (0, ast_exports.isParenthesized)(1, node, sourceCode);
			}
			/**
			* Determines if a node is surrounded by parentheses twice.
			* @param node The node to be checked.
			* @returns True if the node is doubly parenthesised.
			* @private
			*/
			function isParenthesisedTwice(node) {
				return (0, ast_exports.isParenthesized)(2, node, sourceCode);
			}
			/**
			* Determines if a node is surrounded by (potentially) invalid parentheses.
			* @param node The node to be checked.
			* @returns True if the node is incorrectly parenthesised.
			* @private
			*/
			function hasExcessParens(node) {
				return ruleApplies(node) && isParenthesised(node);
			}
			/**
			* Determines if a node that is expected to be parenthesised is surrounded by
			* (potentially) invalid extra parentheses.
			* @param node The node to be checked.
			* @returns True if the node is has an unexpected extra pair of parentheses.
			* @private
			*/
			function hasDoubleExcessParens(node) {
				return ruleApplies(node) && isParenthesisedTwice(node);
			}
			/**
			* Determines if a node that is expected to be parenthesised is surrounded by
			* (potentially) invalid extra parentheses with considering precedence level of the node.
			* If the preference level of the node is not higher or equal to precedence lower limit, it also checks
			* whether the node is surrounded by parentheses twice or not.
			* @param node The node to be checked.
			* @param precedenceLowerLimit The lower limit of precedence.
			* @returns True if the node is has an unexpected extra pair of parentheses.
			* @private
			*/
			function hasExcessParensWithPrecedence(node, precedenceLowerLimit) {
				if (ruleApplies(node) && isParenthesised(node)) {
					if (precedence(node) >= precedenceLowerLimit || isParenthesisedTwice(node)) return true;
				}
				return false;
			}
			/**
			* Determines if a node test expression is allowed to have a parenthesised assignment
			* @param node The node to be checked.
			* @returns True if the assignment can be parenthesised.
			* @private
			*/
			function isCondAssignException(node) {
				return EXCEPT_COND_ASSIGN && node.test && node.test.type === "AssignmentExpression";
			}
			/**
			* Determines if a node is in a return statement
			* @param node The node to be checked.
			* @returns True if the node is in a return statement.
			* @private
			*/
			function isInReturnStatement(node) {
				for (let currentNode = node; currentNode; currentNode = currentNode.parent) if (currentNode.type === "ReturnStatement" || currentNode.type === "ArrowFunctionExpression" && currentNode.body.type !== "BlockStatement") return true;
				return false;
			}
			/**
			* Determines if a constructor function is newed-up with parens
			* @param newExpression The NewExpression node to be checked.
			* @returns True if the constructor is called with parens.
			* @private
			*/
			function isNewExpressionWithParens(newExpression) {
				const lastToken = sourceCode.getLastToken(newExpression);
				const penultimateToken = sourceCode.getTokenBefore(lastToken);
				return newExpression.arguments.length > 0 || (0, ast_exports.isOpeningParenToken)(penultimateToken) && (0, ast_exports.isClosingParenToken)(lastToken) && newExpression.callee.range[1] < newExpression.range[1];
			}
			/**
			* Checks whether a node is a MemberExpression at NewExpression's callee.
			* @param node node to check.
			* @returns True if the node is a MemberExpression at NewExpression's callee. false otherwise.
			*/
			function isMemberExpInNewCallee(node) {
				if (node.type === "MemberExpression") return node.parent.type === "NewExpression" && node.parent.callee === node ? true : "object" in node.parent && node.parent.object === node && isMemberExpInNewCallee(node.parent);
				return false;
			}
			/**
			* Check if a member expression contains a call expression
			* @param node MemberExpression node to evaluate
			* @returns true if found, false if not
			*/
			function doesMemberExpressionContainCallExpression(node) {
				let currentNode = node.object;
				let currentNodeType = node.object.type;
				while (currentNodeType === "MemberExpression") {
					if (!("object" in currentNode)) break;
					currentNode = currentNode.object;
					currentNodeType = currentNode.type;
				}
				return currentNodeType === "CallExpression";
			}
			/**
			* Determines if a node is or contains an assignment expression
			* @param node The node to be checked.
			* @returns True if the node is or contains an assignment expression.
			* @private
			*/
			function containsAssignment(node) {
				if (node.type === "AssignmentExpression") return true;
				if (node.type === "ConditionalExpression" && (node.consequent.type === "AssignmentExpression" || node.alternate.type === "AssignmentExpression")) return true;
				if ("left" in node && (node.left && node.left.type === "AssignmentExpression" || node.right && node.right.type === "AssignmentExpression")) return true;
				return false;
			}
			/**
			* Determines if a node is contained by or is itself a return statement and is allowed to have a parenthesised assignment
			* @param node The node to be checked.
			* @returns True if the assignment can be parenthesised.
			* @private
			*/
			function isReturnAssignException(node) {
				if (!EXCEPT_RETURN_ASSIGN || !isInReturnStatement(node)) return false;
				if (node.type === "ReturnStatement") return node.argument && containsAssignment(node.argument);
				if (node.type === "ArrowFunctionExpression" && node.body.type !== "BlockStatement") return containsAssignment(node.body);
				return containsAssignment(node);
			}
			/**
			* Determines if a node following a [no LineTerminator here] restriction is
			* surrounded by (potentially) invalid extra parentheses.
			* @param token The token preceding the [no LineTerminator here] restriction.
			* @param node The node to be checked.
			* @returns True if the node is incorrectly parenthesised.
			* @private
			*/
			function hasExcessParensNoLineTerminator(token, node) {
				if ((0, ast_exports.isTokenOnSameLine)(token, node)) return hasExcessParens(node);
				return hasDoubleExcessParens(node);
			}
			/**
			* Determines whether a node should be preceded by an additional space when removing parens
			* @param node node to evaluate; must be surrounded by parentheses
			* @returns `true` if a space should be inserted before the node
			* @private
			*/
			function requiresLeadingSpace(node) {
				const leftParenToken = sourceCode.getTokenBefore(node);
				const tokenBeforeLeftParen = sourceCode.getTokenBefore(leftParenToken, { includeComments: true });
				const tokenAfterLeftParen = sourceCode.getTokenAfter(leftParenToken, { includeComments: true });
				return tokenBeforeLeftParen && tokenBeforeLeftParen.range[1] === leftParenToken.range[0] && leftParenToken.range[1] === tokenAfterLeftParen.range[0] && !canTokensBeAdjacent(tokenBeforeLeftParen, tokenAfterLeftParen);
			}
			/**
			* Determines whether a node should be followed by an additional space when removing parens
			* @param node node to evaluate; must be surrounded by parentheses
			* @returns `true` if a space should be inserted after the node
			* @private
			*/
			function requiresTrailingSpace(node) {
				const nextTwoTokens = sourceCode.getTokensAfter(node, { count: 2 });
				const rightParenToken = nextTwoTokens[0];
				const tokenAfterRightParen = nextTwoTokens[1];
				const tokenBeforeRightParen = sourceCode.getLastToken(node);
				return rightParenToken && tokenAfterRightParen && !sourceCode.isSpaceBetween(rightParenToken, tokenAfterRightParen) && !canTokensBeAdjacent(tokenBeforeRightParen, tokenAfterRightParen);
			}
			/**
			* Determines if a given expression node is an IIFE
			* @param node The node to check
			* @returns `true` if the given node is an IIFE
			*/
			function isIIFE(node) {
				const maybeCallNode = skipChainExpression(node);
				return maybeCallNode.type === "CallExpression" && maybeCallNode.callee.type === "FunctionExpression";
			}
			/**
			* Determines if the given node can be the assignment target in destructuring or the LHS of an assignment.
			* This is to avoid an autofix that could change behavior because parsers mistakenly allow invalid syntax,
			* such as `(a = b) = c` and `[(a = b) = c] = []`. Ideally, this function shouldn't be necessary.
			* @param [node] The node to check
			* @returns `true` if the given node can be a valid assignment target
			*/
			function canBeAssignmentTarget(node) {
				return !!(node && (node.type === "Identifier" || node.type === "MemberExpression"));
			}
			/**
			* Checks if the left-hand side of an assignment is an identifier, the operator is one of
			* `=`, `&&=`, `||=` or `??=` and the right-hand side is an anonymous class or function.
			*
			* As per https://tc39.es/ecma262/#sec-assignment-operators-runtime-semantics-evaluation, an
			* assignment involving one of the operators `=`, `&&=`, `||=` or `??=` where the right-hand
			* side is an anonymous class or function and the left-hand side is an *unparenthesized*
			* identifier has different semantics than other assignments.
			* Specifically, when an expression like `foo = function () {}` is evaluated, `foo.name`
			* will be set to the string "foo", i.e. the identifier name. The same thing does not happen
			* when evaluating `(foo) = function () {}`.
			* Since the parenthesizing of the identifier in the left-hand side is significant in this
			* special case, the parentheses, if present, should not be flagged as unnecessary.
			* @param node an AssignmentExpression node.
			* @returns `true` if the left-hand side of the assignment is an identifier, the
			* operator is one of `=`, `&&=`, `||=` or `??=` and the right-hand side is an anonymous
			* class or function; otherwise, `false`.
			*/
			function isAnonymousFunctionAssignmentException({ left, operator, right }) {
				if (left.type === "Identifier" && [
					"=",
					"&&=",
					"||=",
					"??="
				].includes(operator)) {
					const rhsType = right.type;
					if (rhsType === "ArrowFunctionExpression") return true;
					if ((rhsType === "FunctionExpression" || rhsType === "ClassExpression") && !right.id) return true;
				}
				return false;
			}
			/**
			* Checks if a node is fixable.
			* A node is fixable if removing a single pair of surrounding parentheses does not turn it
			* into a directive after fixing other nodes.
			* Almost all nodes are fixable, except if all of the following conditions are met:
			* The node is a string Literal
			* It has a single pair of parentheses
			* It is the only child of an ExpressionStatement
			* @param node The node to evaluate.
			* @returns Whether or not the node is fixable.
			* @private
			*/
			function isFixable(node) {
				if (node.type !== "Literal" || typeof node.value !== "string") return true;
				if (isParenthesisedTwice(node)) return true;
				return !isTopLevelExpressionStatement(node.parent);
			}
			/**
			* Report the node
			* @param node node to evaluate
			* @private
			*/
			function report(node) {
				const leftParenToken = sourceCode.getTokenBefore(node);
				const rightParenToken = sourceCode.getTokenAfter(node);
				if (!isParenthesisedTwice(node)) {
					if (tokensToIgnore.has(sourceCode.getFirstToken(node))) return;
					if (isIIFE(node) && !("callee" in node && isParenthesised(node.callee))) return;
					if (ALLOW_PARENS_AFTER_COMMENT_PATTERN) {
						const commentsBeforeLeftParenToken = sourceCode.getCommentsBefore(leftParenToken);
						const totalCommentsBeforeLeftParenTokenCount = commentsBeforeLeftParenToken.length;
						const ignorePattern = new RegExp(ALLOW_PARENS_AFTER_COMMENT_PATTERN, "u");
						if (totalCommentsBeforeLeftParenTokenCount > 0 && ignorePattern.test(commentsBeforeLeftParenToken[totalCommentsBeforeLeftParenTokenCount - 1].value)) return;
					}
				}
				/**
				* Finishes reporting
				* @private
				*/
				function finishReport() {
					context.report({
						node,
						loc: leftParenToken.loc,
						messageId: "unexpected",
						fix: isFixable(node) ? (fixer) => {
							const parenthesizedSource = sourceCode.text.slice(leftParenToken.range[1], rightParenToken.range[0]);
							return fixer.replaceTextRange([leftParenToken.range[0], rightParenToken.range[1]], (requiresLeadingSpace(node) ? " " : "") + parenthesizedSource + (requiresTrailingSpace(node) ? " " : ""));
						} : null
					});
				}
				if (reportsBuffer) {
					reportsBuffer.reports.push({
						node,
						finishReport
					});
					return;
				}
				finishReport();
			}
			/**
			* Evaluate a argument of the node.
			* @param node node to evaluate
			* @private
			*/
			function checkArgumentWithPrecedence(node) {
				if ("argument" in node && node.argument && hasExcessParensWithPrecedence(node.argument, precedence(node))) report(node.argument);
			}
			/**
			* Evaluate binary logicals
			* @param node node to evaluate
			* @private
			*/
			function checkBinaryLogical(node) {
				const isLeftTypeAssertion = (0, ast_exports.isTypeAssertion)(node.left);
				const isRightTypeAssertion = (0, ast_exports.isTypeAssertion)(node.right);
				if (isLeftTypeAssertion && isRightTypeAssertion) return;
				const rule = (n) => {
					const prec = precedence(n);
					const leftPrecedence = precedence(n.left);
					const rightPrecedence = precedence(n.right);
					const isExponentiation = n.operator === "**";
					const shouldSkipLeft = NESTED_BINARY && (n.left.type === "BinaryExpression" || n.left.type === "LogicalExpression");
					const shouldSkipRight = NESTED_BINARY && (n.right.type === "BinaryExpression" || n.right.type === "LogicalExpression");
					if (!shouldSkipLeft && hasExcessParens(n.left)) {
						if (!(["AwaitExpression", "UnaryExpression"].includes(n.left.type) && isExponentiation) && !isMixedLogicalAndCoalesceExpressions(n.left, n) && !(n.parent.type === "ReturnStatement" && n.parent.loc.start.line !== n.left.loc.start.line && !isParenthesised(n)) && (leftPrecedence > prec || leftPrecedence === prec && !isExponentiation) || isParenthesisedTwice(n.left)) report(n.left);
					}
					if (!shouldSkipRight && hasExcessParens(n.right)) {
						if (!isMixedLogicalAndCoalesceExpressions(n.right, n) && (rightPrecedence > prec || rightPrecedence === prec && isExponentiation) || isParenthesisedTwice(n.right)) report(n.right);
					}
				};
				if (isLeftTypeAssertion) return rule({
					...node,
					left: {
						...node.left,
						type: AST_NODE_TYPES.SequenceExpression
					}
				});
				if (isRightTypeAssertion) return rule({
					...node,
					right: {
						...node.right,
						type: AST_NODE_TYPES.SequenceExpression
					}
				});
				return rule(node);
			}
			function checkCallNew(node) {
				const rule = (node$1) => {
					const callee = node$1.callee;
					if (hasExcessParensWithPrecedence(callee, precedence(node$1))) {
						if (hasDoubleExcessParens(callee) || !(isIIFE(node$1) || callee.type === "NewExpression" && !isNewExpressionWithParens(callee) && !(node$1.type === "NewExpression" && !isNewExpressionWithParens(node$1)) || node$1.type === "NewExpression" && callee.type === "MemberExpression" && doesMemberExpressionContainCallExpression(callee) || (!("optional" in node$1) || !node$1.optional) && callee.type === "ChainExpression")) report(node$1.callee);
					}
					node$1.arguments.filter((arg) => hasExcessParensWithPrecedence(arg, PRECEDENCE_OF_ASSIGNMENT_EXPR)).forEach(report);
				};
				if ((0, ast_exports.isTypeAssertion)(node.callee)) return rule({
					...node,
					callee: {
						...node.callee,
						type: AST_NODE_TYPES.SequenceExpression
					}
				});
				if (node.typeArguments && node.arguments.length === 1 && sourceCode.getTokenAfter(node.callee, ast_exports.isOpeningParenToken) !== sourceCode.getTokenBefore(node.arguments[0], ast_exports.isOpeningParenToken)) return rule({
					...node,
					arguments: [{
						...node.arguments[0],
						type: AST_NODE_TYPES.SequenceExpression
					}]
				});
				return rule(node);
			}
			/**
			* Check the parentheses around the super class of the given class definition.
			* @param node The node of class declarations to check.
			*/
			function checkClass(node) {
				if (!node.superClass) return;
				/**
				* If `node.superClass` is a LeftHandSideExpression, parentheses are extra.
				* Otherwise, parentheses are needed.
				*/
				const hasExtraParens = precedence(node.superClass) > PRECEDENCE_OF_UPDATE_EXPR ? hasExcessParens(node.superClass) : hasDoubleExcessParens(node.superClass);
				if (hasExtraParens) report(node.superClass);
			}
			/**
			* Checks the parentheses for an ExpressionStatement or ExportDefaultDeclaration
			* @param node The ExpressionStatement.expression or ExportDefaultDeclaration.declaration node
			*/
			function checkExpressionOrExportStatement(node) {
				const firstToken = isParenthesised(node) ? sourceCode.getTokenBefore(node) : sourceCode.getFirstToken(node);
				const secondToken = sourceCode.getTokenAfter(firstToken, ast_exports.isNotOpeningParenToken);
				const thirdToken = secondToken ? sourceCode.getTokenAfter(secondToken) : null;
				const tokenAfterClosingParens = secondToken ? sourceCode.getTokenAfter(secondToken, ast_exports.isNotClosingParenToken) : null;
				if ((0, ast_exports.isOpeningParenToken)(firstToken) && ((0, ast_exports.isOpeningBraceToken)(secondToken) || isKeywordToken(secondToken) && (secondToken.value === "function" || secondToken.value === "class" || secondToken.value === "let" && tokenAfterClosingParens && ((0, ast_exports.isOpeningBracketToken)(tokenAfterClosingParens) || tokenAfterClosingParens.type === "Identifier")) || secondToken && secondToken.type === "Identifier" && secondToken.value === "async" && isKeywordToken(thirdToken) && thirdToken.value === "function")) tokensToIgnore.add(secondToken);
				const hasExtraParens = node.parent.type === "ExportDefaultDeclaration" ? hasExcessParensWithPrecedence(node, PRECEDENCE_OF_ASSIGNMENT_EXPR) : hasExcessParens(node);
				if (hasExtraParens) report(node);
			}
			function checkUnaryUpdate(node) {
				if ((0, ast_exports.isTypeAssertion)(node.argument)) return checkArgumentWithPrecedence({
					...node,
					argument: {
						...node.argument,
						type: AST_NODE_TYPES.SequenceExpression
					}
				});
				return checkArgumentWithPrecedence(node);
			}
			return {
				ArrayExpression(node) {
					node.elements.map((element) => (0, ast_exports.isTypeAssertion)(element) ? {
						...element,
						type: AST_NODE_TYPES.FunctionExpression
					} : element).filter((e) => !!e && hasExcessParensWithPrecedence(e, PRECEDENCE_OF_ASSIGNMENT_EXPR)).forEach(report);
				},
				ArrayPattern(node) {
					node.elements.filter((e) => !!e && canBeAssignmentTarget(e) && hasExcessParens(e)).forEach(report);
				},
				ArrowFunctionExpression(node) {
					if ((0, ast_exports.isTypeAssertion)(node.body)) return;
					if (isReturnAssignException(node)) return;
					if (node.body.type === "ConditionalExpression" && IGNORE_ARROW_CONDITIONALS) return;
					if (node.body.type === "BlockStatement") return;
					const firstBodyToken = sourceCode.getFirstToken(node.body, ast_exports.isNotOpeningParenToken);
					const tokenBeforeFirst = sourceCode.getTokenBefore(firstBodyToken);
					if ((0, ast_exports.isOpeningParenToken)(tokenBeforeFirst) && (0, ast_exports.isOpeningBraceToken)(firstBodyToken)) tokensToIgnore.add(firstBodyToken);
					if (hasExcessParensWithPrecedence(node.body, PRECEDENCE_OF_ASSIGNMENT_EXPR)) report(node.body);
				},
				AssignmentExpression(node) {
					if (canBeAssignmentTarget(node.left) && hasExcessParens(node.left) && (!isAnonymousFunctionAssignmentException(node) || isParenthesisedTwice(node.left))) report(node.left);
					if (!isReturnAssignException(node) && hasExcessParensWithPrecedence(node.right, precedence(node))) report(node.right);
				},
				AssignmentPattern(node) {
					const { left, right } = node;
					if (canBeAssignmentTarget(left) && hasExcessParens(left)) report(left);
					if (right && hasExcessParensWithPrecedence(right, PRECEDENCE_OF_ASSIGNMENT_EXPR)) report(right);
				},
				AwaitExpression(node) {
					if ((0, ast_exports.isTypeAssertion)(node.argument)) return checkArgumentWithPrecedence({
						...node,
						argument: {
							...node.argument,
							type: AST_NODE_TYPES.SequenceExpression
						}
					});
					return checkArgumentWithPrecedence(node);
				},
				BinaryExpression(node) {
					if (reportsBuffer && node.operator === "in") reportsBuffer.inExpressionNodes.push(node);
					checkBinaryLogical(node);
				},
				"CallExpression": checkCallNew,
				ClassDeclaration(node) {
					if (node.superClass?.type === AST_NODE_TYPES.TSAsExpression) return checkClass({
						...node,
						superClass: {
							...node.superClass,
							type: AST_NODE_TYPES.SequenceExpression
						}
					});
					return checkClass(node);
				},
				ClassExpression(node) {
					if (node.superClass?.type === AST_NODE_TYPES.TSAsExpression) return checkClass({
						...node,
						superClass: {
							...node.superClass,
							type: AST_NODE_TYPES.SequenceExpression
						}
					});
					return checkClass(node);
				},
				ConditionalExpression(node) {
					const rule = (node$1) => {
						if (isReturnAssignException(node$1)) return;
						const availableTypes = new Set(["BinaryExpression", "LogicalExpression"]);
						if (!(EXCEPT_COND_TERNARY && availableTypes.has(node$1.test.type)) && !(ALLOW_NESTED_TERNARY && ["ConditionalExpression"].includes(node$1.test.type)) && !isCondAssignException(node$1) && hasExcessParensWithPrecedence(node$1.test, precedence({
							type: "LogicalExpression",
							operator: "||"
						}))) report(node$1.test);
						if (!(EXCEPT_COND_TERNARY && availableTypes.has(node$1.consequent.type)) && !(ALLOW_NESTED_TERNARY && ["ConditionalExpression"].includes(node$1.consequent.type)) && hasExcessParensWithPrecedence(node$1.consequent, PRECEDENCE_OF_ASSIGNMENT_EXPR)) report(node$1.consequent);
						if (!(EXCEPT_COND_TERNARY && availableTypes.has(node$1.alternate.type)) && !(ALLOW_NESTED_TERNARY && ["ConditionalExpression"].includes(node$1.alternate.type)) && hasExcessParensWithPrecedence(node$1.alternate, PRECEDENCE_OF_ASSIGNMENT_EXPR)) report(node$1.alternate);
					};
					if ((0, ast_exports.isTypeAssertion)(node.test)) return rule({
						...node,
						test: {
							...node.test,
							type: AST_NODE_TYPES.SequenceExpression
						}
					});
					if ((0, ast_exports.isTypeAssertion)(node.consequent)) return rule({
						...node,
						consequent: {
							...node.consequent,
							type: AST_NODE_TYPES.SequenceExpression
						}
					});
					if ((0, ast_exports.isTypeAssertion)(node.alternate)) return rule({
						...node,
						alternate: {
							...node.alternate,
							type: AST_NODE_TYPES.SequenceExpression
						}
					});
					return rule(node);
				},
				DoWhileStatement(node) {
					if (hasExcessParens(node.test) && !isCondAssignException(node)) report(node.test);
				},
				ExportDefaultDeclaration(node) {
					checkExpressionOrExportStatement(node.declaration);
				},
				ExpressionStatement(node) {
					checkExpressionOrExportStatement(node.expression);
				},
				ForInStatement(node) {
					if ((0, ast_exports.isTypeAssertion)(node.right)) return;
					if (node.left.type !== "VariableDeclaration") {
						const firstLeftToken = sourceCode.getFirstToken(node.left, ast_exports.isNotOpeningParenToken);
						if (firstLeftToken.value === "let" && (0, ast_exports.isOpeningBracketToken)(sourceCode.getTokenAfter(firstLeftToken, ast_exports.isNotClosingParenToken))) tokensToIgnore.add(firstLeftToken);
					}
					if (hasExcessParens(node.left)) report(node.left);
					if (hasExcessParens(node.right)) report(node.right);
				},
				ForOfStatement(node) {
					if (node.left.type !== "VariableDeclaration") {
						const firstLeftToken = sourceCode.getFirstToken(node.left, ast_exports.isNotOpeningParenToken);
						if (firstLeftToken.value === "let") tokensToIgnore.add(firstLeftToken);
					}
					if (hasExcessParens(node.left)) report(node.left);
					if (!(0, ast_exports.isTypeAssertion)(node.right) && hasExcessParensWithPrecedence(node.right, PRECEDENCE_OF_ASSIGNMENT_EXPR)) report(node.right);
				},
				ForStatement(node) {
					if (node.test && hasExcessParens(node.test) && !isCondAssignException(node) && !(0, ast_exports.isTypeAssertion)(node.test)) report(node.test);
					if (node.update && hasExcessParens(node.update) && !(0, ast_exports.isTypeAssertion)(node.update)) report(node.update);
					if (node.init && !(0, ast_exports.isTypeAssertion)(node.init)) {
						if (node.init.type !== "VariableDeclaration") {
							const firstToken = sourceCode.getFirstToken(node.init, ast_exports.isNotOpeningParenToken);
							if (firstToken.value === "let" && (0, ast_exports.isOpeningBracketToken)(sourceCode.getTokenAfter(firstToken, ast_exports.isNotClosingParenToken))) tokensToIgnore.add(firstToken);
						}
						startNewReportsBuffering();
						if (hasExcessParens(node.init)) report(node.init);
					}
				},
				"ForStatement > *.init:exit": function(node) {
					if ((0, ast_exports.isTypeAssertion)(node)) return;
					/**
					* Removing parentheses around `in` expressions might change semantics and cause errors.
					*
					* For example, this valid for loop:
					*      for (let a = (b in c); ;);
					* after removing parentheses would be treated as an invalid for-in loop:
					*      for (let a = b in c; ;);
					*/
					if (reportsBuffer?.reports.length) reportsBuffer.inExpressionNodes.forEach((inExpressionNode) => {
						const path = pathToDescendant(node, inExpressionNode);
						let nodeToExclude = null;
						for (let i = 0; i < path.length; i++) {
							const pathNode = path[i];
							if (i < path.length - 1) {
								const nextPathNode = path[i + 1];
								if (isSafelyEnclosingInExpression(pathNode, nextPathNode)) return;
							}
							if (isParenthesised(pathNode)) if (isInCurrentReportsBuffer(pathNode)) {
								if (isParenthesisedTwice(pathNode))
 /**
								* This node is parenthesised twice, it certainly has at least one pair of `extra` parentheses.
								* If the --fix option is on, the current fixing iteration will remove only one pair of parentheses.
								* The remaining pair is safely enclosing the 'in' expression.
								*/
								return;
								if (!nodeToExclude) nodeToExclude = pathNode;
							} else return;
						}
						if (nodeToExclude) removeFromCurrentReportsBuffer(nodeToExclude);
					});
					endCurrentReportsBuffering();
				},
				IfStatement(node) {
					if (hasExcessParens(node.test) && !isCondAssignException(node)) report(node.test);
				},
				ImportExpression(node) {
					const { source } = node;
					if (source.type === "SequenceExpression") {
						if (hasDoubleExcessParens(source)) report(source);
					} else if (hasExcessParens(source)) report(source);
				},
				"LogicalExpression": checkBinaryLogical,
				MemberExpression(node) {
					const rule = (node$1) => {
						const shouldAllowWrapOnce = isMemberExpInNewCallee(node$1) && doesMemberExpressionContainCallExpression(node$1);
						const nodeObjHasExcessParens = shouldAllowWrapOnce ? hasDoubleExcessParens(node$1.object) : hasExcessParens(node$1.object) && !(isImmediateFunctionPrototypeMethodCall(node$1.parent) && "callee" in node$1.parent && node$1.parent.callee === node$1 && IGNORE_FUNCTION_PROTOTYPE_METHODS);
						if (nodeObjHasExcessParens && precedence(node$1.object) >= precedence(node$1) && (node$1.computed || !(isDecimalInteger(node$1.object) || node$1.object.type === "Literal" && "regex" in node$1.object && node$1.object.regex))) report(node$1.object);
						if (nodeObjHasExcessParens && node$1.object.type === "CallExpression") report(node$1.object);
						if (nodeObjHasExcessParens && !IGNORE_NEW_IN_MEMBER_EXPR && node$1.object.type === "NewExpression" && isNewExpressionWithParens(node$1.object)) report(node$1.object);
						if (nodeObjHasExcessParens && node$1.optional && node$1.object.type === "ChainExpression") report(node$1.object);
						if (node$1.computed && hasExcessParens(node$1.property)) report(node$1.property);
					};
					if ((0, ast_exports.isTypeAssertion)(node.object)) return rule({
						...node,
						object: {
							...node.object,
							type: AST_NODE_TYPES.SequenceExpression
						}
					});
					if ((0, ast_exports.isTypeAssertion)(node.property)) return rule({
						...node,
						property: {
							...node.property,
							type: AST_NODE_TYPES.FunctionExpression
						}
					});
					return rule(node);
				},
				MethodDefinition(node) {
					if (!node.computed) return;
					if (hasExcessParensWithPrecedence(node.key, PRECEDENCE_OF_ASSIGNMENT_EXPR)) report(node.key);
				},
				"NewExpression": checkCallNew,
				ObjectExpression(node) {
					node.properties.filter((property) => property.type === "Property" && property.value && hasExcessParensWithPrecedence(property.value, PRECEDENCE_OF_ASSIGNMENT_EXPR)).forEach((property) => report(property.value));
				},
				ObjectPattern(node) {
					node.properties.filter((property) => {
						const value = property.value;
						return value && canBeAssignmentTarget(value) && hasExcessParens(value);
					}).forEach((property) => report(property.value));
				},
				Property(node) {
					if (node.computed) {
						const { key } = node;
						if (key && hasExcessParensWithPrecedence(key, PRECEDENCE_OF_ASSIGNMENT_EXPR)) report(key);
					}
				},
				PropertyDefinition(node) {
					if (node.computed && hasExcessParensWithPrecedence(node.key, PRECEDENCE_OF_ASSIGNMENT_EXPR)) report(node.key);
					if (node.value && hasExcessParensWithPrecedence(node.value, PRECEDENCE_OF_ASSIGNMENT_EXPR)) report(node.value);
				},
				RestElement(node) {
					const argument = node.argument;
					if (canBeAssignmentTarget(argument) && hasExcessParens(argument)) report(argument);
				},
				ReturnStatement(node) {
					const returnToken = sourceCode.getFirstToken(node);
					if (isReturnAssignException(node)) return;
					if (node.argument && returnToken && hasExcessParensNoLineTerminator(returnToken, node.argument) && !(node.argument.type === "Literal" && "regex" in node.argument && node.argument.regex)) report(node.argument);
				},
				SequenceExpression(node) {
					const precedenceOfNode = precedence(node);
					node.expressions.filter((e) => hasExcessParensWithPrecedence(e, precedenceOfNode)).forEach(report);
				},
				SpreadElement(node) {
					if ((0, ast_exports.isTypeAssertion)(node.argument)) return;
					if (ALLOW_NODES_IN_SPREAD && ALLOW_NODES_IN_SPREAD.has(node.argument.type)) return;
					if (!hasExcessParensWithPrecedence(node.argument, PRECEDENCE_OF_ASSIGNMENT_EXPR)) return;
					report(node.argument);
				},
				SwitchCase(node) {
					if (node.test && !(0, ast_exports.isTypeAssertion)(node.test) && hasExcessParens(node.test)) report(node.test);
				},
				SwitchStatement(node) {
					if (hasExcessParens(node.discriminant)) report(node.discriminant);
				},
				TemplateLiteral(node) {
					node.expressions.filter((e) => e && hasExcessParens(e)).forEach(report);
				},
				ThrowStatement(node) {
					if (!node.argument || (0, ast_exports.isTypeAssertion)(node.argument)) return;
					const throwToken = sourceCode.getFirstToken(node);
					if (!throwToken) return;
					if (hasExcessParensNoLineTerminator(throwToken, node.argument)) report(node.argument);
				},
				"UnaryExpression": checkUnaryUpdate,
				UpdateExpression(node) {
					if ((0, ast_exports.isTypeAssertion)(node.argument)) return checkUnaryUpdate(node);
					if (node.prefix) checkArgumentWithPrecedence(node);
					else {
						const { argument } = node;
						const operatorToken = sourceCode.getLastToken(node);
						if ((0, ast_exports.isTokenOnSameLine)(argument, operatorToken)) checkArgumentWithPrecedence(node);
						else if (hasDoubleExcessParens(argument)) report(argument);
					}
				},
				VariableDeclarator(node) {
					const rule = (node$1) => {
						if (node$1.init && hasExcessParensWithPrecedence(node$1.init, PRECEDENCE_OF_ASSIGNMENT_EXPR) && !(node$1.init.type === "Literal" && "regex" in node$1.init && node$1.init.regex)) report(node$1.init);
					};
					if ((0, ast_exports.isTypeAssertion)(node.init)) return rule({
						...node,
						type: AST_NODE_TYPES.VariableDeclarator,
						init: {
							...node.init,
							type: AST_NODE_TYPES.FunctionExpression
						}
					});
					return rule(node);
				},
				WhileStatement(node) {
					if (hasExcessParens(node.test) && !isCondAssignException(node)) report(node.test);
				},
				WithStatement(node) {
					if (hasExcessParens(node.object)) report(node.object);
				},
				YieldExpression(node) {
					if (!node.argument || (0, ast_exports.isTypeAssertion)(node.argument)) return;
					const yieldToken = sourceCode.getFirstToken(node);
					if (precedence(node.argument) >= precedence(node) && yieldToken && hasExcessParensNoLineTerminator(yieldToken, node.argument) || hasDoubleExcessParens(node.argument)) report(node.argument);
				},
				TSStringKeyword(node) {
					if (hasExcessParens(node)) report({
						...node,
						type: AST_NODE_TYPES.FunctionExpression
					});
				}
			};
		}
	});
});
export { init_no_extra_parens, no_extra_parens_default };
